@@ -3,10 +3,10 @@ from datetime import date, datetime, timedelta
 from typing import Any, Dict, List, Optional
 
 import requests
-import urllib3
-from icalendar import Calendar
-from icalendar import Event as CalEvent
-from icalendar.prop import vDatetime
+import urllib3  # type: ignore
+from icalendar import Calendar  # type: ignore
+from icalendar import Event as CalEvent  # type: ignore
+from icalendar.prop import vDatetime  # type: ignore
 
 from .event import Event
 from .exceptions import ScheduleFetchError, WUDeadError
@@ -46,14 +46,20 @@ class Schedule:
         """
 
         try:
-            calendar: Calendar = Calendar.from_ical(requests.get(self._url, verify=False, timeout=timeout).text)  # type: ignore
+            calendar: Calendar = Calendar.from_ical(
+                requests.get(self._url, verify=False, timeout=timeout).text  # nosec
+            )  # type: ignore
         except requests.exceptions.ConnectTimeout as e:
             raise WUDeadError(e)
         except Exception as e:
             raise ScheduleFetchError(e)
 
         # create a list of events out of the calendar
-        self.events = [Event(component) for component in calendar.walk() if component.name == "VEVENT"]
+        self.events = [
+            Event.from_calendar(component)
+            for component in calendar.walk()
+            if component.name == "VEVENT"
+        ]
 
         self.first_day = min(self.events, key=lambda e: e.start).start.date()
         self.last_day = max(self.events, key=lambda e: e.start).start.date()
@@ -75,7 +81,32 @@ class Schedule:
         """
         return self.events
 
-    def get_events(self, start_date: Optional[date] = None, end_date: Optional[date] = None) -> List[Dict[str, Any]]:
+    def dump_json(self) -> str:
+        """
+        Dump as list of events encoded in json
+
+        :returns: a json string
+        """
+
+        return json.dumps(self.events, default=Schedule.serialize_json)
+
+    def from_json(self, events_json: str) -> None:
+        events = [
+            Event(
+                name=evt["name"],
+                type=evt["type"],
+                teacher=evt["teacher"],
+                location=evt["location"],
+                start=datetime.fromisoformat(evt["start"]),
+                end=datetime.fromisoformat(evt["end"]),
+            )
+            for evt in json.loads(events_json)
+        ]
+        self.load_events(events)
+
+    def get_events(
+        self, start_date: Optional[date] = None, end_date: Optional[date] = None
+    ) -> List[Dict[str, Any]]:
         """
         Get events as a list of objects
 
@@ -107,7 +138,9 @@ class Schedule:
 
             if "wychowanie fizyczne" in event.name.lower():
                 duplicates = [
-                    e for e in self.events if (e is not event and e.start == event.start and e.end == event.end)
+                    e
+                    for e in self.events
+                    if (e is not event and e.start == event.start and e.end == event.end)
                 ]
 
                 if len(duplicates) > 0 and not event.teacher and not event.location:
@@ -119,34 +152,6 @@ class Schedule:
 
         return response
 
-    @staticmethod
-    def format_as_json(events: List[dict]) -> str:
-        """
-        Format existing schedule to json
-        :returns: json string
-        """
-
-        def serialize(o: Any) -> Any:
-            """
-            Serialize function for json.dumps
-
-            Convert date and datetime to isoformat string
-            Convert Event object to its dict representation
-
-            :param o: object to serialize
-            :returns: serialized object string
-            """
-            if isinstance(o, datetime):
-                return o.isoformat()
-
-            if isinstance(o, date):
-                return o.isoformat()
-
-            if isinstance(o, Event):
-                return o.__dict__
-
-        return json.dumps(events, default=serialize)
-
     def get_json(self, start_date: date = None, end_date: date = None) -> str:
         """
         Get the schedule as json
@@ -159,6 +164,17 @@ class Schedule:
         schedule = self.get_events(start_date, end_date)
         return self.format_as_json(schedule)
 
+    def to_ical(self, start_date: Optional[date] = None, end_date: Optional[date] = None) -> bytes:
+        """
+        Get the schedule as iCalendar file
+
+        :param start_date: Schedule start date - optional, defaults to schedule start date
+        :param end_date: Schedule end date - optional, defaults to schedule end date
+        :returns: ics file
+        """
+        schedule = self.get_events(start_date, end_date)
+        return self.format_as_ical(schedule)
+
     @staticmethod
     def format_as_ical(events: List[dict]) -> bytes:
         """
@@ -189,13 +205,30 @@ class Schedule:
 
         return cal.to_ical()
 
-    def get_ical(self, start_date: Optional[date] = None, end_date: Optional[date] = None) -> bytes:
+    @staticmethod
+    def serialize_json(o: Any) -> Any:
         """
-        Get the schedule as iCalendar file
+        Serialize function for json.dumps
 
-        :param start_date: Schedule start date - optional, defaults to schedule start date
-        :param end_date: Schedule end date - optional, defaults to schedule end date
-        :returns: ics file
+        Convert date and datetime to isoformat string
+        Convert Event object to its dict representation
+
+        :param o: object to serialize
+        :returns: serialized object string
         """
-        schedule = self.get_events(start_date, end_date)
-        return self.format_as_ical(schedule)
+        if isinstance(o, datetime):
+            return o.isoformat()
+
+        if isinstance(o, date):
+            return o.isoformat()
+
+        if isinstance(o, Event):
+            return o.__dict__
+
+    @staticmethod
+    def format_as_json(events: List[dict]) -> str:
+        """
+        Format existing schedule to json
+        :returns: json string
+        """
+        return json.dumps(events, default=Schedule.serialize_json, indent=2)
